package cn.com.qmth.stmms.biz.exam.query;

import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;

import cn.com.qmth.stmms.biz.common.BaseQuery;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.enums.SubjectiveStatus;
import io.swagger.annotations.ApiModelProperty;

public class ExamStudentSearchQuery extends BaseQuery<ExamStudent> {

    @ApiModelProperty(value = "筛选 0-无;1-客观0分，主观有分;2-客观有分，主观0分")
    private Integer filter;

    private Integer userId;

    private Role role;

    private Integer examId;

    private Integer schoolId;

    private String campusName;

    @ApiModelProperty(value = "姓名")
    private String name;

    // private String examNumberIn;
    @ApiModelProperty(value = "准考证号数组")
    private List<String> examNumbers;

    @ApiModelProperty(value = "准考证号")
    private String examNumber;

    private String secretNumber;

    @ApiModelProperty(value = "学号")
    private String studentCode;

    @ApiModelProperty(value = "科目")
    private String subjectCode;

    private String batchCode;

    private String packageCode;

    private String subjectLevel;

    private String subjectCategory;

    @ApiModelProperty(value = "考点")
    private String examSite;

    @ApiModelProperty(value = "考场")
    private String examRoom;

    private Double objectiveScore;

    private Double objectiveScoreGt;

    private Double objectiveScoreLt;

    private Set<SubjectiveStatus> statusSet;

    private Double subjectiveScore;

    private Double subjectiveScoreGt;

    private Double subjectiveScoreLt;

    @ApiModelProperty(value = "是否上传")
    private Boolean upload;

    @ApiModelProperty(value = "是否缺考")
    private Boolean absent;

    private Boolean manualAbsent;

    private Boolean exception;

    private Boolean uploadTimeNotNull;

    private Date minUploadTime;

    private Date maxUploadTime;

    private String campusNameIn;

    @ApiModelProperty(value = "科目数组")
    private List<String> subjectCodes;

    @ApiModelProperty(value = "考点数组")
    private List<String> examSites;

    private String campusNameNotIn;

    private String subjectCodeNotIn;

    private String examSiteNotIn;

    private String ids;

    @ApiModelProperty(value = "总分开始")
    private Double startScore;

    @ApiModelProperty(value = "总分截止")
    private Double endScore;

    @ApiModelProperty(value = "是否违纪")
    private Boolean breach;

    private Boolean trial;

    @ApiModelProperty(value = "学院")
    private String college;

    @ApiModelProperty(value = "班级")
    private String className;

    @ApiModelProperty(value = "任课老师")
    private String teacher;

    private String paperType;

    private List<Integer> studentIds;

    private Integer inspectorId;

    private String inspectorName;

    private Integer sheetCount;

    // 是否已复核
    private Boolean inspected;

    private Double questionScore;

    private Boolean questionScoreEmpty;

    // 不可重复
    private Boolean inspectUnrepeated;

    private Boolean objectiveFill;

    // 有客观题的科目
    private List<String> objectiveCodes;

    public ExamStudentSearchQuery() {
        super();
        this.statusSet = new HashSet<>();
    }

    public void orderByExamNumber() {
        setSort(new Sort(Direction.ASC, "examNumber"));
    }

    public void orderByExamIdDesc() {
        setSort(new Sort(Direction.DESC, "examId"));
    }

    public void orderByInspectTimeDesc() {
        setSort(new Sort(Direction.DESC, "inspectTime"));
    }

    public void orderById() {
        setSort(new Sort(Direction.ASC, "id"));
    }

    public Integer getExamId() {
        return examId;
    }

    public void setExamId(Integer examId) {
        this.examId = examId;
    }

    public Integer getSchoolId() {
        return schoolId;
    }

    public void setSchoolId(Integer schoolId) {
        this.schoolId = schoolId;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getSubjectCode() {
        return subjectCode;
    }

    public void setSubjectCode(String subjectCode) {
        this.subjectCode = subjectCode;
    }

    public String getExamNumber() {
        return examNumber;
    }

    public void setExamNumber(String examNumber) {
        this.examNumber = examNumber;
    }

    public String getSecretNumber() {
        return secretNumber;
    }

    public void setSecretNumber(String secretNumber) {
        this.secretNumber = secretNumber;
    }

    public String getStudentCode() {
        return studentCode;
    }

    public void setStudentCode(String studentCode) {
        this.studentCode = studentCode;
    }

    public Boolean getUpload() {
        return upload;
    }

    public void setUpload(Boolean upload) {
        this.upload = upload;
    }

    public Boolean getAbsent() {
        return absent;
    }

    public void setAbsent(Boolean absent) {
        this.absent = absent;
    }

    public String getCampusName() {
        return campusName;
    }

    public void setCampusName(String campusName) {
        this.campusName = campusName;
    }

    public String getBatchCode() {
        return batchCode;
    }

    public void setBatchCode(String batchCode) {
        this.batchCode = batchCode;
    }

    public String getPackageCode() {
        return packageCode;
    }

    public void setPackageCode(String packageCode) {
        this.packageCode = packageCode;
    }

    public Boolean getException() {
        return exception;
    }

    public void setException(Boolean exception) {
        this.exception = exception;
    }

    public Double getObjectiveScore() {
        return objectiveScore;
    }

    public void setObjectiveScore(Double objectiveScore) {
        this.objectiveScore = objectiveScore;
    }

    public Double getObjectiveScoreGt() {
        return objectiveScoreGt;
    }

    public void setObjectiveScoreGt(Double objectiveScoreGt) {
        this.objectiveScoreGt = objectiveScoreGt;
    }

    public Double getSubjectiveScore() {
        return subjectiveScore;
    }

    public void setSubjectiveScore(Double subjectiveScore) {
        this.subjectiveScore = subjectiveScore;
    }

    public Double getSubjectiveScoreGt() {
        return subjectiveScoreGt;
    }

    public void setSubjectiveScoreGt(Double subjectiveScoreGt) {
        this.subjectiveScoreGt = subjectiveScoreGt;
    }

    public String getSubjectLevel() {
        return subjectLevel;
    }

    public void setSubjectLevel(String subjectLevel) {
        this.subjectLevel = subjectLevel;
    }

    public String getSubjectCategory() {
        return subjectCategory;
    }

    public void setSubjectCategory(String subjectCategory) {
        this.subjectCategory = subjectCategory;
    }

    public Boolean getUploadTimeNotNull() {
        return uploadTimeNotNull;
    }

    public void setUploadTimeNotNull(Boolean uploadTimeNotNull) {
        this.uploadTimeNotNull = uploadTimeNotNull;
    }

    public Date getMinUploadTime() {
        return minUploadTime;
    }

    public void setMinUploadTime(Date minUploadTime) {
        this.minUploadTime = minUploadTime;
    }

    public Date getMaxUploadTime() {
        return maxUploadTime;
    }

    public void setMaxUploadTime(Date maxUploadTime) {
        this.maxUploadTime = maxUploadTime;
    }

    // public String getExamNumberIn() {
    // return examNumberIn;
    // }
    //
    // public void setExamNumberIn(String examNumberIn) {
    // this.examNumberIn = examNumberIn;
    // }

    public Double getObjectiveScoreLt() {
        return objectiveScoreLt;
    }

    public void setObjectiveScoreLt(Double objectiveScoreLt) {
        this.objectiveScoreLt = objectiveScoreLt;
    }

    public Double getSubjectiveScoreLt() {
        return subjectiveScoreLt;
    }

    public void setSubjectiveScoreLt(Double subjectiveScoreLt) {
        this.subjectiveScoreLt = subjectiveScoreLt;
    }

    public String getCampusNameIn() {
        return campusNameIn;
    }

    public void setCampusNameIn(String campusNameIn) {
        this.campusNameIn = campusNameIn;
    }

    public String getCampusNameNotIn() {
        return campusNameNotIn;
    }

    public void setCampusNameNotIn(String campusNameNotIn) {
        this.campusNameNotIn = campusNameNotIn;
    }

    public String getSubjectCodeNotIn() {
        return subjectCodeNotIn;
    }

    public void setSubjectCodeNotIn(String subjectCodeNotIn) {
        this.subjectCodeNotIn = subjectCodeNotIn;
    }

    public String getIds() {
        return ids;
    }

    public void setIds(String ids) {
        this.ids = ids;
    }

    public Double getStartScore() {
        return startScore;
    }

    public void setStartScore(Double startScore) {
        this.startScore = startScore;
    }

    public Double getEndScore() {
        return endScore;
    }

    public void setEndScore(Double endScore) {
        this.endScore = endScore;
    }

    public Boolean getBreach() {
        return breach;
    }

    public void setBreach(Boolean breach) {
        this.breach = breach;
    }

    public Boolean getManualAbsent() {
        return manualAbsent;
    }

    public void setManualAbsent(Boolean manualAbsent) {
        this.manualAbsent = manualAbsent;
    }

    public String getCollege() {
        return college;
    }

    public void setCollege(String college) {
        this.college = college;
    }

    public String getClassName() {
        return className;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public String getTeacher() {
        return teacher;
    }

    public void setTeacher(String teacher) {
        this.teacher = teacher;
    }

    public String getPaperType() {
        return paperType;
    }

    public void setPaperType(String paperType) {
        this.paperType = paperType;
    }

    public Set<SubjectiveStatus> getStatusSet() {
        return statusSet;
    }

    public void clearStatus() {
        this.statusSet.clear();
    }

    public void addStatus(SubjectiveStatus status) {
        this.statusSet.add(status);
    }

    public List<Integer> getStudentIds() {
        return studentIds;
    }

    public void setStudentIds(List<Integer> studentIds) {
        this.studentIds = studentIds;
    }

    public Integer getInspectorId() {
        return inspectorId;
    }

    public void setInspectorId(Integer inspectorId) {
        this.inspectorId = inspectorId;
    }

    public String getExamSite() {
        return examSite;
    }

    public void setExamSite(String examSite) {
        this.examSite = examSite;
    }

    public String getExamRoom() {
        return examRoom;
    }

    public void setExamRoom(String examRoom) {
        this.examRoom = examRoom;
    }

    public String getExamSiteNotIn() {
        return examSiteNotIn;
    }

    public void setExamSiteNotIn(String examSiteNotIn) {
        this.examSiteNotIn = examSiteNotIn;
    }

    public Integer getSheetCount() {
        return sheetCount;
    }

    public void setSheetCount(Integer sheetCount) {
        this.sheetCount = sheetCount;
    }

    public Boolean getInspected() {
        return inspected;
    }

    public void setInspected(Boolean inspected) {
        this.inspected = inspected;
    }

    public String getInspectorName() {
        return inspectorName;
    }

    public void setInspectorName(String inspectorName) {
        this.inspectorName = inspectorName;
    }

    public Double getQuestionScore() {
        return questionScore;
    }

    public void setQuestionScore(Double questionScore) {
        this.questionScore = questionScore;
    }

    public Boolean getQuestionScoreEmpty() {
        return questionScoreEmpty;
    }

    public void setQuestionScoreEmpty(Boolean questionScoreEmpty) {
        this.questionScoreEmpty = questionScoreEmpty;
    }

    public Boolean getInspectUnrepeated() {
        return inspectUnrepeated;
    }

    public void setInspectUnrepeated(Boolean inspectUnrepeated) {
        this.inspectUnrepeated = inspectUnrepeated;
    }

    public Role getRole() {
        return role;
    }

    public void setRole(Role role) {
        this.role = role;
    }

    public Integer getUserId() {
        return userId;
    }

    public void setUserId(Integer userId) {
        this.userId = userId;
    }

    public Boolean getTrial() {
        return trial;
    }

    public void setTrial(Boolean trial) {
        this.trial = trial;
    }

    public Boolean getObjectiveFill() {
        return objectiveFill;
    }

    public void setObjectiveFill(Boolean objectiveFill) {
        this.objectiveFill = objectiveFill;
    }

    public List<String> getObjectiveCodes() {
        return objectiveCodes;
    }

    public void setObjectiveCodes(List<String> objectiveCodes) {
        this.objectiveCodes = objectiveCodes;
    }

    public Integer getFilter() {
        return filter;
    }

    public void setFilter(Integer filter) {
        this.filter = filter;
    }

    public List<String> getExamNumbers() {
        return examNumbers;
    }

    public void setExamNumbers(List<String> examNumbers) {
        this.examNumbers = examNumbers;
    }

    public List<String> getSubjectCodes() {
        return subjectCodes;
    }

    public void setSubjectCodes(List<String> subjectCodes) {
        this.subjectCodes = subjectCodes;
    }

    public List<String> getExamSites() {
        return examSites;
    }

    public void setExamSites(List<String> examSites) {
        this.examSites = examSites;
    }

}
