package cn.com.qmth.stmms.api.controller.admin;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.google.common.collect.Lists;
import com.qmth.boot.core.collection.PageResult;

import cn.com.qmth.stmms.admin.thread.MarkerResetThread;
import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.config.service.impl.SystemCache;
import cn.com.qmth.stmms.biz.exam.bean.MarkerVo;
import cn.com.qmth.stmms.biz.exam.bean.ResultMessage;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.query.MarkerSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerClassService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.exception.StatusException;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.mark.service.RejectHistoryService;
import cn.com.qmth.stmms.biz.mark.service.TrialService;
import cn.com.qmth.stmms.biz.school.model.School;
import cn.com.qmth.stmms.biz.school.service.SchoolService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.MarkStatus;
import cn.com.qmth.stmms.common.enums.MarkerExcelError;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.enums.UserSource;
import cn.com.qmth.stmms.common.utils.EncryptUtils;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.ImportExcel;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

@Api(tags = "评卷员管理")
@Controller("adminMarkerController")
@RequestMapping("/api/admin/exam/marker")
public class MarkerController extends BaseApiController {

    protected static Logger log = LoggerFactory.getLogger(MarkerController.class);

    public static final String USER_PASSWORD = "123456";

    @Autowired
    private UserService userService;

    @Autowired
    private ExamService examService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private MarkService markService;

    @Autowired
    private LockService lockService;

    @Autowired
    private MarkerClassService classService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private TrialService trialService;

    @Qualifier("task-executor")
    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @Autowired
    private SchoolService schoolService;

    @Autowired
    private RejectHistoryService rejectHistoryService;

    @Autowired
    private SystemCache systemCache;

    @Value("${mark.group.delete}")
    private String markDeleteCode;

    @ApiOperation(value = "评卷员分页查询")
    @Logging(menu = "评卷员查询", type = LogType.QUERY)
    @ResponseBody
    @RequestMapping(value = "list", method = RequestMethod.POST)
    public PageResult<MarkerVo> getPage(MarkerSearchQuery query) {
        ApiUser wu = getApiUser();
        int examId = getSessionExamId();
        query.setExamId(examId);
        List<ExamSubject> subjectList = getExamSubject(examId, wu);
        if (StringUtils.isBlank(query.getSubjectCode()) && wu.getRole() != Role.SCHOOL_ADMIN) {
            List<String> subjectCodeIn = new ArrayList<String>();
            subjectList.forEach(e -> subjectCodeIn.add(e.getCode()));
            query.setSubjectCodeIn(StringUtils.join(subjectCodeIn, ","));
        }
        query.orderById();
        query = markerService.findByQuery(query);
        List<MarkerVo> ret = new ArrayList<>();
        for (Marker marker : query.getResult()) {
            marker.setSubject(subjectService.find(marker.getExamId(), marker.getSubjectCode()));
            MarkGroup group = groupService.findOne(examId, marker.getSubjectCode(), marker.getGroupNumber());
            int percent = group.getLibraryCount() > 0
                    ? (int) (group.getMarkedCount() * 100.00 / group.getLibraryCount())
                    : 0;
            if (group.getMarkedCount() > 0 && percent == 0) {
                percent = 1;
            } else if (group.getLeftCount() > 0 && percent == 100) {
                percent = 99;
            }
            group.setQuestionList(questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(examId,
                    marker.getSubjectCode(), false, group.getNumber()));
            group.setPercent(percent);
            marker.setGroup(group);
            marker.setMarkedCount(markService.markedCount(marker));
            marker.setCurrentCount(markService.applyCount(marker));
            marker.setUser(userService.findById(marker.getUserId()));
            marker.setReseting(lockService.isLocked(LockType.MARKER_RESET, marker.getId()));
            marker.setClassCount(classService.countByUserIdAndExamId(marker.getUserId(), examId));
            ret.add(MarkerVo.of(marker));
        }
        return PageUtil.of(ret, query);
    }

    @ApiOperation(value = "重置是否弹窗确认")
    @ResponseBody
    @RequestMapping(value = "delete-check", method = RequestMethod.POST)
    public Boolean deleteCheck(MarkerSearchQuery query) {
        int examId = getSessionExamId();
        Exam exam = examService.findById(examId);
        School school = schoolService.findById(exam.getSchoolId());
        return systemCache.isGroupDeleteWarn() ? true : school.isGroupDeleteCheck();
    }

    @ApiOperation(value = "评卷员下拉列表")
    @RequestMapping(value = "query", method = RequestMethod.POST)
    @ResponseBody
    public List<MarkerVo> query(@RequestParam String subjectCode, @RequestParam(required = false) Integer groupNumber) {
        int examId = getSessionExamId();
        List<MarkerVo> array = new ArrayList<>();
        MarkerSearchQuery query = new MarkerSearchQuery();
        query.setExamId(examId);
        query.setSubjectCode(subjectCode);
        query.setGroupNumber(groupNumber);
        query.setPageSize(Integer.MAX_VALUE);
        List<Marker> list = markerService.findByQuery(query).getResult();
        for (Marker marker : list) {
            MarkerVo obj = new MarkerVo();
            obj.setId(marker.getId());
            User user = userService.findById(marker.getUserId());
            obj.setLoginName(user.getLoginName());
            obj.setName(user.getName());
            array.add(obj);
        }
        return array;
    }

    @ApiOperation(value = "回收")
    @Logging(menu = "评卷员回收任务", type = LogType.UPDATE)
    @RequestMapping(value = "release", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER, Role.COLLEGE_ADMIN })
    public ResultMessage release(@RequestParam Integer[] ids) {
        for (Integer id : ids) {
            Marker marker = markerService.findById(id);
            if (marker != null) {
                try {
                    lockService.waitlock(LockType.MARKER, marker.getId());
                    markService.releaseByMarker(marker);
                } catch (Exception e) {
                    log.error("release marker error", e);
                    throw new RuntimeException("回收出错", e);
                } finally {
                    lockService.unlock(LockType.MARKER, marker.getId());
                }
            }
        }
        return resultOk();
    }

    @ApiOperation(value = "评卷员重置")
    @Logging(menu = "评卷员重置", type = LogType.UPDATE)
    @RequestMapping(value = "reset", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire(Role.SCHOOL_ADMIN)
    public ResultMessage resetMarker(@RequestParam Integer id,
            @ApiParam("授权码") @RequestParam(required = false) String deleteCode) {
        Marker marker = markerService.findById(id);
        int examId = getSessionExamId();
        if (marker != null) {
            Exam exam = examService.findById(examId);
            School school = schoolService.findById(exam.getSchoolId());
            boolean warn = systemCache.isGroupDeleteWarn() ? true : school.isGroupDeleteCheck();
            MarkGroup group = groupService.findOne(examId, marker.getSubjectCode(), marker.getGroupNumber());
            if (warn && group.getLibraryCount() != 0 && group.getLeftCount() == 0
                    && !markDeleteCode.equals(deleteCode)) {
                throw new StatusException("重置失败，授权码不正确");
            }
            if (group.getStatus() == MarkStatus.FINISH) {
                throw new StatusException("重置失败，关联分组已关闭");
            }
            if (lockService.trylock(LockType.MARKER_RESET, marker.getId())) {
                taskExecutor.submit(new MarkerResetThread(marker, group, markService, libraryService, trialService,
                        markerService, groupService, lockService));
                return resultOk();
            } else {
                throw new StatusException("评卷员正在重置");
            }
        } else {
            throw new StatusException("重置失败，该评卷员不存在");
        }
    }

    @ApiOperation(value = "评卷员禁用、启用")
    @Logging(menu = "评卷员禁用/启用", type = LogType.UPDATE)
    @RequestMapping(value = "toggle", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER, Role.COLLEGE_ADMIN })
    public ResultMessage toggle(@RequestParam Integer[] ids, @RequestParam Boolean enable) {
        for (Integer id : ids) {
            Marker marker = markerService.findById(id);
            if (marker != null) {
                marker.setEnable(enable);
                markerService.save(marker);
            }
        }
        return resultOk();
    }

    @ApiOperation(value = "重置密码")
    @Logging(menu = "评卷员重置密码", type = LogType.UPDATE)
    @RequestMapping(value = "reset-password", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage reSetPassword(@RequestParam Integer[] ids, @RequestParam String password) {
        for (Integer id : ids) {
            Marker marker = markerService.findById(id);
            User user = userService.findById(marker.getUserId());
            if (user == null) {
                continue;
            }
            password = StringEscapeUtils.unescapeHtml(password);
            user.setPassword(EncryptUtils.md5(password));
            user.setRandomPassword(null);
            user = userService.save(user);
        }
        return resultOk();
    }

    @ApiOperation(value = "设置任务数量")
    @Logging(menu = "评卷员设置任务数量", type = LogType.UPDATE)
    @RequestMapping(value = "set-task-count", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage setTaskCount(@RequestParam Integer[] ids, @RequestParam Integer taskCount) {
        for (Integer id : ids) {
            Marker marker = markerService.findById(id);
            if (marker != null) {
                marker.setTopCount(taskCount);
                marker = markerService.save(marker);
            }
        }
        return resultOk();
    }

    @ResponseBody
    @ApiOperation(value = "导入模版下载")
    @RequestMapping(value = "template", method = RequestMethod.POST)
    public void importFileTemplate(HttpServletResponse response) {
        String fileName = "评卷员数据导入模板.xlsx";
        List<Marker> list = Lists.newArrayList();
        list.add(new Marker());
        try {
            new ExportExcel("评卷员数据", Marker.class, 2).setDataList(list).write(response, fileName).dispose();
        } catch (IOException e) {
            throw new RuntimeException("导入模板下载失败", e);
        }
    }

    @ResponseBody
    @ApiOperation(value = "导入")
    @Logging(menu = "评卷员导入", type = LogType.IMPORT_FILE)
    @RequestMapping(value = "import", method = RequestMethod.POST)
    public ResultMessage importFile(MultipartFile file, @RequestParam String subjectCode) {
        ResultMessage ret = new ResultMessage();
        int examId = getSessionExamId();
        ApiUser wu = RequestUtils.getApiUser();
        Integer schoolId = wu.getUser().getSchoolId();
        try {
            int successNum = 0;
            int failureNum = 0;
            StringBuilder failureMsg = new StringBuilder();
            ImportExcel ei = new ImportExcel(file, 1, 0);
            List<Marker> list = ei.getDataList(Marker.class);

            Map<String, User> userMap = new HashMap<String, User>();
            Map<String, ExamSubject> subjectMap = new HashMap<String, ExamSubject>();
            Map<String, MarkGroup> groupMap = new HashMap<String, MarkGroup>();
            Map<String, Marker> markerMap = new HashMap<String, Marker>();
            for (Marker marker : list) {
                if (StringUtils.isBlank(marker.getLoginName())) {
                    failureMsg.append("\r\n评卷员 " + marker.getLoginName() + "," + marker.getSubjectCode()
                            + MarkerExcelError.NAMENULL.getName());
                    failureNum++;
                    continue;
                }
                ExamSubject examSubject = subjectMap.get(marker.getSubjectCode());
                if (examSubject == null) {
                    examSubject = subjectService.find(examId, marker.getSubjectCode());
                    if (examSubject == null) {
                        failureMsg.append("\r\n评卷员 " + marker.getLoginName() + "," + marker.getSubjectCode()
                                + MarkerExcelError.CODENULL.getName());
                        failureNum++;
                        continue;
                    }
                }
                MarkGroup markGroup = groupMap.get(marker.getSubjectCode() + "-" + marker.getGroupNumber());
                if (markGroup == null) {
                    markGroup = groupService.findOne(examId, marker.getSubjectCode(), marker.getGroupNumber());
                    if (markGroup == null) {
                        failureMsg.append("\r\n评卷员 " + marker.getLoginName() + "," + marker.getSubjectCode()
                                + MarkerExcelError.GROUPNULL.getName());
                        failureNum++;
                        continue;
                    }
                }
                User user = userMap.get(marker.getLoginName().trim());
                if (user == null) {
                    user = userService.findByLoginName(marker.getLoginName().trim());
                    if (user != null && !schoolId.equals(user.getSchoolId())) {
                        failureMsg.append("\r\n评卷员 " + marker.getLoginName() + "," + marker.getSubjectCode()
                                + MarkerExcelError.NAMEUSED.getName());
                        failureNum++;
                        continue;
                    }
                    if (user != null && !user.getRole().equals(Role.MARKER)) {
                        failureMsg.append("\r\n评卷员 " + marker.getLoginName() + "," + marker.getSubjectCode()
                                + MarkerExcelError.NAMENOTMARKER.getName());
                        failureNum++;
                        continue;
                    }
                    if (user == null) {
                        user = getUser(userMap, marker.getLoginName().trim(), schoolId);
                    }
                }
                subjectMap.put(examSubject.getCode(), examSubject);
                groupMap.put(markGroup.getSubjectCode() + "-" + markGroup.getNumber(), markGroup);
                marker = getMarker(markerMap, examId, examSubject.getCode(), markGroup.getNumber(), user.getId());
            }
            List<Marker> saveList = new ArrayList<Marker>();
            for (Marker m : markerMap.values()) {
                saveList.add(m);
            }
            successNum = markerService.batchSave(saveList);
            if (failureNum > 0) {
                failureMsg.insert(0, "，失败 " + failureNum + " 条记录！");
            }
            String message = "已成功导入 " + successNum + " 条评卷员 ";
            ret.setSuccess(true);
            ret.setMessage(message + failureMsg);
            RequestUtils.setLog(message);
        } catch (Exception e) {
            log.error("Batch import marker error!", e);
            throw new RuntimeException("导入评卷员失败", e);
        }
        return ret;
    }

    private Marker getMarker(Map<String, Marker> markerMap, int examId, String subjectCode, Integer groupNumber,
            Integer userId) {
        String key = examId + "-" + subjectCode + "-" + groupNumber + "-" + userId;
        Marker marker = markerMap.get(key);
        if (marker == null) {
            marker = markerService.findByExamAndSubjectAndNumberAndUserId(examId, subjectCode, groupNumber, userId);
        }
        if (marker == null) {
            marker = new Marker();
            marker.setExamId(examId);
            marker.setSubjectCode(subjectCode);
            marker.setGroupNumber(groupNumber);
            marker.setUserId(userId);
            marker.setEnable(true);
        }
        markerMap.put(key, marker);
        return marker;
    }

    private User getUser(Map<String, User> saveMap, String loginName, Integer schoolId) {
        User user = saveMap.get(loginName);
        if (user == null) {
            user = new User();
            user.setLoginName(loginName);
            user.setName(loginName);
            user.setPassword(EncryptUtils.md5(USER_PASSWORD));
            user.setRole(Role.MARKER);
            user.setSource(UserSource.INTERNAL);
            user.setSchoolId(schoolId);
            user.setEnable(true);
            user.setCreatedTime(new Date());
            userService.save(user);
            saveMap.put(loginName, user);
        }
        return user;
    }

    @ResponseBody
    @ApiOperation(value = "评卷员绑定")
    @Logging(menu = "评卷员绑定", type = LogType.ADD)
    @RequestMapping(value = "add", method = RequestMethod.POST)
    public ResultMessage add(@RequestParam String subjectCode, @RequestParam Integer groupNumber,
            @RequestParam Integer[] userIds) {
        int examId = getSessionExamId();
        MarkGroup group = groupService.findOne(examId, subjectCode, groupNumber);
        if (group != null) {
            markerService.save(group, userIds);
            return resultOk();
        } else {
            throw new StatusException("科目分组不存在");
        }
    }

    @ApiOperation(value = "解绑")
    @ResponseBody
    @RequestMapping(value = "delete", method = RequestMethod.POST)
    public ResultMessage delete(@RequestParam Integer markerId) {
        Marker marker = markerService.findById(markerId);
        if (marker == null) {
            throw new StatusException("未找到评卷员信息");
        }
        long count = libraryService.countByMarker(markerId);
        long trialCount = trialService.countByMarkerIdAndMarkerScoreIsNotNull(markerId);
        long rejectCount = rejectHistoryService.countByMarkerId(markerId);
        int examId = marker.getExamId();
        String subjectCode = marker.getSubjectCode();
        Integer groupNumber = marker.getGroupNumber();
        if (count > 0 || trialCount > 0 || rejectCount > 0) {
            throw new StatusException("删除评卷员失败，该评卷员已开始评卷");
        } else {
            try {
                lockService.watch(LockType.EXAM_SUBJECT, examId, subjectCode);
                lockService.watch(LockType.GROUP, examId, subjectCode, groupNumber);
                lockService.waitlock(LockType.MARKER, markerId);
                markService.deleteMarker(marker);
            } catch (Exception e) {
                log.error("release marker error", e);
                throw new RuntimeException("解绑出错", e);
            } finally {
                lockService.unlock(LockType.MARKER, markerId);
                lockService.unwatch(LockType.GROUP, examId, subjectCode, groupNumber);
                lockService.unwatch(LockType.EXAM_SUBJECT, examId, subjectCode);
            }
        }
        return resultOk();
    }

    @ApiOperation(value = "评卷员班级名")
    @ResponseBody
    @RequestMapping(value = "class/name", method = RequestMethod.POST)
    public Set<String> classList(@RequestParam Integer markerId) {
        Marker marker = markerService.findById(markerId);
        if (marker != null) {
            marker.setUser(userService.findById(marker.getUserId()));
            return classService.findClassNameByUserIdAndExamId(marker.getUserId(), marker.getExamId());
        } else {
            throw new StatusException("评卷员不存在");
        }
    }

    @ApiOperation(value = "评卷员班级绑定")
    @Logging(menu = "评卷员班级绑定", type = LogType.UPDATE)
    @ResponseBody
    @RequestMapping(value = "class/add", method = RequestMethod.POST)
    public ResultMessage classSave(@RequestParam Integer markerId, @RequestParam(required = false) String[] classes) {
        Marker marker = markerService.findById(markerId);
        if (marker != null) {
            classService.save(marker.getUserId(), marker.getExamId(), classes);
            return resultOk();
        } else {
            throw new StatusException("评卷员不存在");
        }
    }

    @ApiOperation(value = "清空班级")
    @Logging(menu = "评卷员班级清空", type = LogType.DELETE)
    @ResponseBody
    @RequestMapping(value = "class/delete", method = RequestMethod.POST)
    public ResultMessage classDelete(@RequestParam Integer markerId) {
        Marker marker = markerService.findById(markerId);
        if (marker != null) {
            classService.deleteByUserIdAndExamId(marker.getUserId(), marker.getExamId());
            return resultOk();
        } else {
            throw new StatusException("评卷员不存在");
        }
    }

}
