package cn.com.qmth.stmms.api.controller.admin;

import java.math.BigDecimal;
import java.util.*;

import javax.servlet.http.HttpServletRequest;

import cn.com.qmth.stmms.biz.exam.bean.ResultMessage;
import io.swagger.annotations.Api;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.stmms.admin.thread.QuestionDeleteThread;
import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.SelectiveGroup;
import cn.com.qmth.stmms.biz.exam.service.*;
import cn.com.qmth.stmms.biz.exam.service.query.ExamQuestionSearchQuery;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.enums.*;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONObject;

@Api(tags = "题目管理")
@Controller("adminQuestionController")
@RequestMapping("/api/admin/question")
public class QuestionController extends BaseApiController {

    protected static final Logger log = LoggerFactory.getLogger(QuestionController.class);

    private static final String NULL_PAPER_TYPE_PLACEHOLDER = "#";

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamService examService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private InspectHistoryService inspectHistoryService;

    @Qualifier("task-executor")
    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @Autowired
    private LockService lockService;

    @Autowired
    private MarkService markService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private SelectiveGroupService selectiveGroupService;

    @ApiOperation(value = "科目详情-题目列表")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    @ResponseBody
    public List<ExamQuestion> list(ExamQuestionSearchQuery query) {
        int examId = getSessionExamId();
        ExamSubject subject = subjectService.find(examId, query.getSubjectCode());
        if (subject == null) {
            throw new StatusException("没有找到科目");
        }
        List<ExamQuestion> list = questionService.findByQuery(query).getResult();
        List<MarkGroup> groups = groupService.findByExamAndSubject(examId, query.getSubjectCode());
        Map<Integer, MarkGroup> maps = new HashMap<Integer, MarkGroup>();
        for (MarkGroup markGroup : groups) {
            maps.put(markGroup.getNumber(), markGroup);
        }
        List<SelectiveGroup> selectiveGroups = selectiveGroupService.findByExamIdAndSubjectCode(examId,
                query.getSubjectCode());
        Map<Integer, SelectiveGroup> selectiveMap = new HashMap<Integer, SelectiveGroup>();
        for (SelectiveGroup selectiveGroup : selectiveGroups) {
            selectiveMap.put(selectiveGroup.getMainNumber(), selectiveGroup);
        }
        for (ExamQuestion examQuestion : list) {
            if (examQuestion.getGroupNumber() != null && maps.get(examQuestion.getGroupNumber()).getMarkedCount() > 0) {
                examQuestion.setEnableUpdate(false);
            } else {
                examQuestion.setEnableUpdate(true);
            }
            // 选做题判分策略
            if (selectiveMap.containsKey(examQuestion.getMainNumber())) {
                examQuestion.setEnableUpdate(false);
                examQuestion.setSelective(true);
                examQuestion.setSelectiveIndex(selectiveMap.get(examQuestion.getMainNumber()).getSelectiveIndex());
                examQuestion.setSelectivePart(selectiveMap.get(examQuestion.getMainNumber()).getSelectivePart());
                examQuestion.setScorePolicy(selectiveMap.get(examQuestion.getMainNumber()).getScorePolicy().getValue());
            } else {
                examQuestion.setSelective(false);
            }
            examQuestion.setDeleting(lockService.isLocked(LockType.QUESTION_DELETE, examQuestion.getExamId(),
                    examQuestion.getSubjectCode(), examQuestion.getMainNumber(), examQuestion.getSubNumber()));
        }
        return list;
    }

    @ApiOperation(value = "大题列表")
    @RequestMapping(value = "/main/list", method = RequestMethod.POST)
    @ResponseBody
    public List<ExamQuestion> mainList(ExamQuestionSearchQuery query) {
        int examId = getSessionExamId();
        ExamSubject subject = subjectService.find(examId, query.getSubjectCode());
        if (subject == null) {
            throw new StatusException("没有找到科目");
        }
        List<ExamQuestion> mainList = new LinkedList<ExamQuestion>();
        mainList.addAll(questionService.findMainByExamAndSubjectAndObjective(examId, query.getSubjectCode(), true));
        mainList.addAll(questionService.findMainByExamAndSubjectAndObjective(examId, query.getSubjectCode(), false));
        return mainList;
    }

    @ApiOperation(value = "客观题判分策略")
    @RequestMapping(value = "/objectivePolicy/list", method = RequestMethod.POST)
    @ResponseBody
    public List<ObjectivePolicy> roleList() {
        return Arrays.asList(ObjectivePolicy.values());
    }

    @ApiOperation(value = "题目类型")
    @RequestMapping(value = "/type/list", method = RequestMethod.POST)
    @ResponseBody
    public List<QuestionType> typeList() {
        return Arrays.asList(QuestionType.values());
    }

    @ApiOperation(value = "删除题目")
    @Logging(menu = "删除题目", type = LogType.DELETE)
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage delete(HttpServletRequest request, @RequestParam List<Integer> ids) {
        List<ExamQuestion> list = new ArrayList<ExamQuestion>();
        for (Integer id : ids) {
            ExamQuestion question = questionService.findById(id);
            if (question == null) {
                throw new StatusException("没有找到题目");
            }
            SelectiveGroup selectiveGroup = selectiveGroupService.findOne(question.getExamId(),
                    question.getSubjectCode(), question.getMainNumber());
            if (selectiveGroup != null) {
                throw new StatusException("删除失败，该题目已有分组或选做题存在选做题分组");
            }
            if (!question.isObjective() && question.getGroupNumber() != null) {
                throw new StatusException("删除失败，该题目已有分组或选做题存在选做题分组");
            }
            if (lockService.trylock(LockType.QUESTION_DELETE, question.getExamId(), question.getSubjectCode(),
                    question.getMainNumber(), question.getSubNumber())) {
                list.add(question);
                RequestUtils.setLog(request, "删除题目，科目代码：" + question.getSubjectCode() + " 大题号:"
                        + question.getMainNumber() + " 小题号:" + question.getSubNumber());
            }
        }
        if (!list.isEmpty()) {
            taskExecutor.submit(new QuestionDeleteThread(list, markService, lockService));
        }
        return resultOk();
    }

    @ApiOperation(value = "编辑题目")
    @Logging(menu = "编辑题目", type = LogType.UPDATE)
    @RequestMapping(value = "/edit", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage update(HttpServletRequest request, ExamQuestion question) {
        int examId = getSessionExamId(request);
        ExamQuestion old = questionService.findById(question.getId());
        ExamQuestion newQ = questionService.findByExamAndSubjectAndObjectiveAndMainNumberAndSubNumber(examId,
                question.getSubjectCode(), question.isObjective(), question.getMainNumber(), question.getSubNumber());
        if (old == null || ((old.getMainNumber() != question.getMainNumber()
                || !old.getSubNumber().equals(question.getSubNumber())) && newQ != null)) {
            throw new StatusException("编辑失败，编辑题目不存在或编辑后的题号已存在");
        }
        questionService.updateMainName(question.getId(), question.getName());
        questionService.updateTrackCount(question.getId(),
                question.getTrackCount() == null ? 0 : question.getTrackCount());
        if (!old.isObjective() && old.getGroupNumber() != null) {
            MarkGroup group = groupService.findOne(examId, old.getSubjectCode(), old.getGroupNumber());
            if (group.getMarkedCount() > 0) {
                throw new StatusException("编辑失败，已经开始评卷");
            }
        }
        SelectiveGroup selectiveGroup = selectiveGroupService.findOne(examId, old.getSubjectCode(),
                old.getMainNumber());
        if (!old.isObjective() && selectiveGroup != null) {
            throw new StatusException("编辑失败，已经存在选做题分组");
        }
        List<String> error = new LinkedList<String>();
        if (validate(question, error, new HashMap<Integer, String>())) {
            throw new StatusException(StringUtils.join(error, " "));
        }
        old.setMainNumber(question.getMainNumber());
        old.setSubNumber(question.getSubNumber());
        old.setMainTitle(question.getMainTitle());
        old.setName(question.getName());
        old.setTotalScore(question.getTotalScore());
        old.setIntervalScore(question.getIntervalScore());
        old.setObjectivePolicy(question.getObjectivePolicy());
        old.setAnswer(question.getAnswer().toUpperCase());
        old.setPaperType(question.getPaperType());
        old.setType(question.getType());
        old.setTrackCount(question.getTrackCount() == null ? 0 : question.getTrackCount());
        questionService.saveAndFlush(old);
        questionService.updateMainTitle(examId, question.getSubjectCode(), question.isObjective(),
                question.getMainNumber(), question.getMainTitle());
        subjectService.updateScore(examId, question.getSubjectCode(), question.isObjective(),
                questionService.sumTotalScore(examId, question.getSubjectCode(), question.isObjective()));
        // 更新分组总分
        if (!old.isObjective() && old.getGroupNumber() != null) {
            markerService.logoutByExamIdAndSubjectCodeAndGroupNumber(examId, old.getSubjectCode(),
                    old.getGroupNumber());
            MarkGroup group = groupService.findOne(examId, old.getSubjectCode(), old.getGroupNumber());
            List<ExamQuestion> questionGroup = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(examId,
                    question.getSubjectCode(), false, old.getGroupNumber());
            BigDecimal totalScore = BigDecimal.ZERO;
            for (ExamQuestion q : questionGroup) {
                totalScore = totalScore.add(BigDecimal.valueOf(q.getTotalScore()));
            }
            group.setTotalScore(totalScore.doubleValue());
            groupService.save(group);
        }
        return resultOk();
    }

    private boolean validate(ExamQuestion question, List<String> error, Map<Integer, String> titleMap) {
        if (countDecimalPlaces(question.getTotalScore()) > 2) {
            error.add("有满分为小数超2位的记录");
            return false;
        }
        if (question.isObjective() && StringUtils.isBlank(question.getAnswer())) {
            error.add("有答案为空的记录");
            return false;
        }
        if (question.isObjective() && question.getType() == null) {
            error.add("有题型为空的记录");
            return false;
        }
        if (QuestionType.TRUE_OR_FALSE.equals(question.getType())
                && (!question.getAnswer().contains("A") && !question.getAnswer().contains("B"))) {
            error.add("有判断题标答不为A或B的记录");
            return false;
        }
        if (QuestionType.SINGLE.equals(question.getType()) && question.getAnswer().length() > 1) {
            error.add("有单选题标答大于一个的记录");
            return false;
        }
        String title = titleMap.get(question.getMainNumber());
        if (title != null && !title.equals(question.getMainTitle())) {
            error.add("有名称不一致的记录");
            return false;
        }
        if (question.isObjective()) {
            question.setAnswer(question.getAnswer().toUpperCase());
        }
        return true;
    }

    public int countDecimalPlaces(double number) {
        String num = Double.toString(number);
        int decimalPointIndex = num.indexOf('.');
        if (decimalPointIndex == -1) {
            return 0;
        }
        return num.length() - decimalPointIndex - 1;
    }

    @ApiOperation(value = "新增题目")
    @Logging(menu = "新增题目", type = LogType.ADD)
    @RequestMapping(value = "/question/save", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage save(HttpServletRequest request, ExamQuestion question) {
        int examId = getSessionExamId(request);
        ExamQuestion old = questionService.findByExamAndSubjectAndObjectiveAndMainNumberAndSubNumber(examId,
                question.getSubjectCode(), question.isObjective(), question.getMainNumber(), question.getSubNumber());
        if (old != null) {
            throw new StatusException("新增失败，题号已存在");
        }
        question.setExamId(examId);
        if (StringUtils.isBlank(question.getPaperType())) {
            question.setPaperType(NULL_PAPER_TYPE_PLACEHOLDER);
        }
        if (question.isObjective()) {
            question.setGroupNumber(0);
        } else {
            question.setObjectivePolicy(null);
            question.setAnswer(null);
            question.setType(null);
        }
        if (question.getTrackCount() == null) {
            question.setTrackCount(0);
        }
        Map<Integer, String> titleMap = new HashMap<>();
        List<ExamQuestion> current = questionService.findByExamAndSubjectAndObjective(examId, question.getSubjectCode(),
                question.isObjective());
        for (ExamQuestion q : current) {
            titleMap.put(q.getMainNumber(), q.getMainTitle());
        }
        List<String> error = new LinkedList<String>();
        if (validate(question, error, titleMap)) {
            ResultMessage resultMessage = new ResultMessage();
            resultMessage.setMessage(StringUtils.join(error, " "));
            resultMessage.setSuccess(false);
            return resultMessage;
        }
        questionService.save(question);
        subjectService.updateScore(examId, question.getSubjectCode(), question.isObjective(),
                questionService.sumTotalScore(examId, question.getSubjectCode(), question.isObjective()));
        if (!question.isObjective()) {
            studentService.updateSubjectiveStatusAndScore(examId, question.getSubjectCode(), SubjectiveStatus.UNMARK, 0,
                    null);
            inspectHistoryService.deleteByExamIdAndSubjectCode(examId, question.getSubjectCode());
        }
        return resultOk();
    }
}
