package cn.com.qmth.stmms.api.controller.admin;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exception.StatusException;
import cn.com.qmth.stmms.biz.mark.model.RejectType;
import cn.com.qmth.stmms.biz.mark.service.ProblemHistoryService;
import cn.com.qmth.stmms.biz.mark.service.RejectTypeService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.enums.LogType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "打回卷卷分类管理")
@Controller("adminRejectTypeController")
@RequestMapping("/api/admin/exam/reject/type")
public class RejectTypeController extends BaseApiController {

    protected static Logger log = LoggerFactory.getLogger(RejectTypeController.class);

    @Autowired
    private RejectTypeService rejectTypeService;

    @Autowired
    private ProblemHistoryService historyService;

    @ApiOperation(value = "查询")
    @Logging(menu = "查询打回卷分类", type = LogType.QUERY)
    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public List<RejectType> list(@RequestParam Integer examId) {
        return rejectTypeService.findByExamId(examId);
    }

    @ApiOperation(value = "新增")
    @Logging(menu = "新增打回卷分类", type = LogType.ADD)
    @RequestMapping(value = "save", method = RequestMethod.POST)
    @ResponseBody
    public void save(RejectType problem) {
        if (problem.getExamId() == null) {
            throw new StatusException("examId不能为空");
        }
        if (!StringUtils.isNotBlank(problem.getName())) {
            throw new StatusException("名称不能为空");
        }
        Integer examId = problem.getExamId();
        List<RejectType> list = rejectTypeService.findByExamId(examId);
        if (list.size() >= 10) {
            throw new StatusException("打回卷类型不能超过10个");
        }
        problem.setCustom(true);
        problem = rejectTypeService.save(problem);
    }

    @ApiOperation(value = "查询信息")
    @RequestMapping(value = "find", method = RequestMethod.POST)
    @ResponseBody
    public RejectType find(@PathVariable Integer id) {
        return rejectTypeService.findById(id);
    }

    @ApiOperation(value = "修改")
    @Logging(menu = "编辑打回卷分类", type = LogType.UPDATE)
    @RequestMapping(value = "update", method = RequestMethod.POST)
    @ResponseBody
    public void update(RejectType problem) {
        ApiUser user = getApiUser();
        RejectType old = rejectTypeService.findById(problem.getId());
        if (user.isSchoolAdmin() && StringUtils.isNotBlank(problem.getName())) {
            old.setName(problem.getName());
            rejectTypeService.save(old);
        }
    }

    @ApiOperation(value = "删除")
    @Logging(menu = "删除打回卷分类", type = LogType.DELETE)
    @RequestMapping(value = "delete", method = RequestMethod.POST)
    @ResponseBody
    public void delete(@RequestParam Integer id) {
        int examId = getSessionExamId();
        long historyCount = historyService.countByExamIdAndProblemId(examId, id);// TODO
                                                                                 // 修改校验，待打回卷修改完后处理这段代码
        if (historyCount > 0) {
            throw new StatusException("该打回卷类型已被使用，不可删除");
        } else {
            rejectTypeService.deleteById(id);
        }
    }
}
