package cn.com.qmth.stmms.api.controller.admin;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.qmth.boot.core.collection.PageResult;

import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.config.model.SystemAuth;
import cn.com.qmth.stmms.biz.config.service.SystemAuthService;
import cn.com.qmth.stmms.biz.config.service.impl.SystemCache;
import cn.com.qmth.stmms.biz.exam.bean.ResultMessage;
import cn.com.qmth.stmms.biz.exam.bean.SchoolDomain;
import cn.com.qmth.stmms.biz.exam.bean.SchoolStatusVo;
import cn.com.qmth.stmms.biz.exam.bean.SchoolVo;
import cn.com.qmth.stmms.biz.exception.StatusException;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.school.model.School;
import cn.com.qmth.stmms.biz.school.query.SchoolSearchQuery;
import cn.com.qmth.stmms.biz.school.service.SchoolService;
import cn.com.qmth.stmms.biz.school.thread.OrgSyncThread;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.enums.SystemAuthType;
import cn.com.qmth.stmms.common.enums.UserSource;
import cn.com.qmth.stmms.common.utils.EncryptUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "学校管理")
@Controller("adminSchoolController")
@RequestMapping("/api/admin/sys/school")
public class SchoolController extends BaseApiController {

    @Autowired
    private SchoolService schoolService;

    @Autowired
    private UserService userService;

    @Qualifier("task-executor")
    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @Autowired
    private LockService lockService;

    @Autowired
    private SystemAuthService authService;

    @Autowired
    private SystemCache authCache;

    @ApiOperation(value = "学校状态")
    @ResponseBody
    @RequestMapping(value = "status", method = RequestMethod.POST)
    public SchoolStatusVo status() {
        SchoolStatusVo ret = new SchoolStatusVo();
        ret.setRunning(lockService.isLocked(LockType.SCHOOL, ""));
        boolean enableUpadteOrg = false;
        SystemAuth systemAuth = authService.findOne();
        if (systemAuth != null && SystemAuthType.ONLINE.equals(systemAuth.getType()) && authCache.isAuth()) {
            enableUpadteOrg = true;
        }
        ret.setSyncEnable(enableUpadteOrg);
        ret.setDoubleTrack(authCache.isDoubleTrack());
        ret.setGroupDeleteWarn(authCache.isGroupDeleteWarn());
        return ret;
    }

    @Logging(menu = "学校查询", type = LogType.QUERY)
    @ApiOperation(value = "学校查询")
    @RequestMapping(value = "list", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<SchoolVo> list(SchoolSearchQuery query) {
        query = schoolService.findByQuery(query);
        List<SchoolVo> ret = new ArrayList<>();
        for (School e : query.getResult()) {
            ret.add(SchoolVo.of(e));
        }

        return PageUtil.of(ret, query);
    }

    @ApiOperation(value = "学校信息查询")
    @RequestMapping(value = "info", method = RequestMethod.POST)
    @ResponseBody
    public SchoolDomain info(@RequestParam Integer schoolId) {
        School school = schoolService.findById(schoolId);
        if (school == null) {
            throw new StatusException("未找到学校信息");
        }
        return SchoolDomain.of(school);
    }

    @Logging(menu = "新增（修改）学校", type = LogType.UPDATE)
    @ApiOperation(value = "新增（修改）学校")
    @RequestMapping(value = "save", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage save(HttpServletRequest request, SchoolDomain domain) {
        School school = SchoolDomain.of(domain);
        School previous = null;
        if (school.getId() != null) {
            previous = schoolService.findById(school.getId());
        }
        if (previous == null) {
            school.setEnable(true);
            school.setDoubleTrack(false);
            school.resetAccessKeyAndSecret();
            school = schoolService.save(school);
        } else {
            if (authCache.isDoubleTrack()) {
                previous.setDoubleTrack(school.isDoubleTrack());
            }
            previous.setName(school.getName());
            previous.setProvince(school.getProvince());
            previous.setCity(school.getCity());
            previous.setAddress(school.getAddress());
            if (!authCache.isGroupDeleteWarn()) {
                previous.setGroupDeleteCheck(school.isGroupDeleteCheck());
            }
            previous.setDescription(school.getDescription());
            if (StringUtils.isBlank(previous.getAccessKey()) || StringUtils.isBlank(previous.getAccessSecret())) {
                previous.resetAccessKeyAndSecret();
            }
            schoolService.save(previous);
        }
        return resultOk();
    }

    @ApiOperation(value = "学校管理员信息")
    @RequestMapping(value = "admin/info", method = RequestMethod.POST)
    @ResponseBody
    public User adminInfo(@RequestParam Integer schoolId) {
        School school = schoolService.findById(schoolId);
        if (school != null) {
            User user = userService.findSchoolAdmin(schoolId);
            if (user == null) {
                user = new User();
                user.setSchoolId(schoolId);
            }
            return user;
        } else {
            throw new StatusException("未找到学校信息");
        }
    }

    @Logging(menu = "编辑学校管理员", type = LogType.UPDATE)
    @ApiOperation(value = "编辑学校管理员")
    @RequestMapping(value = "admin/save", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage adminSave(User user) {
        if (user.getSchoolId() == null) {
            throw new StatusException("SchoolId不能为空");
        }
        School school = schoolService.findById(user.getSchoolId());
        if (school == null) {
            throw new StatusException("未找到学校信息");
        }
        User previous = userService.findSchoolAdmin(user.getSchoolId());
        if (previous != null) {
            previous.setLoginName(user.getLoginName());
            previous.setName(user.getName());
            if (StringUtils.isNotBlank(user.getPassword())) {
                previous.setPassword(EncryptUtils.md5(user.getPassword()));
            }
            if (checkLoginName(previous)) {
                userService.save(previous);
                return resultOk();
            } else {
                throw new StatusException("登录名重复");
            }
        } else if (StringUtils.isBlank(user.getPassword())) {
            throw new StatusException("密码不能为空");
        } else if (checkLoginName(user)) {
            user.setEnable(true);
            user.setRole(Role.SCHOOL_ADMIN);
            user.setSource(UserSource.INTERNAL);
            user.setPassword(EncryptUtils.md5(user.getPassword()));
            userService.save(user);
            return resultOk();
        } else {
            throw new StatusException("登录名重复");
        }
    }

    @ApiOperation(value = "重置密钥")
    @RequestMapping(value = "rest-access", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage restAccess(@RequestParam Integer schoolId) {
        School school = schoolService.findById(schoolId);
        if (school != null) {
            school.resetAccessKeyAndSecret();
            schoolService.save(school);
            return resultOk();
        } else {
            throw new StatusException("未找到学校信息");
        }
    }

    @ApiOperation(value = "机构同步")
    @RequestMapping(value = "sync", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage updateOrg() {
        if (lockService.trylock(LockType.SCHOOL, "")) {
            OrgSyncThread thread = new OrgSyncThread(schoolService, authCache);
            taskExecutor.submit(thread);
            return resultOk();
        } else {
            throw new StatusException("正在同步中");
        }
    }
}
