package cn.com.qmth.stmms.api.controller.admin;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.qmth.boot.core.collection.PageResult;

import cn.com.qmth.stmms.api.controller.BaseApiController;
import cn.com.qmth.stmms.biz.exam.bean.ResultMessage;
import cn.com.qmth.stmms.biz.exam.bean.ScoreVerifyVo;
import cn.com.qmth.stmms.biz.exam.query.ScoreVerifyQuery;
import cn.com.qmth.stmms.biz.exam.service.ScoreVerifyService;
import cn.com.qmth.stmms.biz.exception.StatusException;
import cn.com.qmth.stmms.biz.utils.PageUtil;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "仲裁管理")
@Controller("adminScoreVerifyController")
@RequestMapping("/api/admin/exam/score/verify")
public class ScoreVerifyController extends BaseApiController {

    protected static Logger log = LoggerFactory.getLogger(ScoreVerifyController.class);

    @Autowired
    private ScoreVerifyService scoreVerifyService;

    @ApiOperation(value = "重置")
    @RequestMapping(value = "reset", method = RequestMethod.POST)
    @ResponseBody
    public ResultMessage reset() {
        Integer examId = getSessionExamId();
        ApiUser wu = getApiUser();
        if (!scoreVerifyService.existByExamIdAndUserId(examId, wu.getUser().getId())) {
            throw new StatusException("没有数据需要重置");
        } else {
            scoreVerifyService.reset(examId, wu.getUser().getId());
            return resultOk();
        }
    }

    @ApiOperation(value = "判断是已有数据")
    @RequestMapping(value = "has-data", method = RequestMethod.POST)
    @ResponseBody
    public Boolean hasDate() {
        Integer examId = getSessionExamId();
        ApiUser wu = getApiUser();
        return scoreVerifyService.existByExamIdAndUserId(examId, wu.getUser().getId());
    }

    @ApiOperation(value = "查询待处理数量")
    @RequestMapping(value = "need-verify-count", method = RequestMethod.POST)
    @ResponseBody
    public Integer needVerifyCount() {
        Integer examId = getSessionExamId();
        ApiUser wu = getApiUser();
        return scoreVerifyService.needVerifyCount(examId, wu.getUser().getId());
    }

    @ApiOperation(value = "分页查询")
    @RequestMapping(value = "page", method = RequestMethod.POST)
    @ResponseBody
    public PageResult<ScoreVerifyVo> page(ScoreVerifyQuery query) {
        Integer examId = getSessionExamId();
        ApiUser wu = getApiUser();
        if (!scoreVerifyService.existByExamIdAndUserId(examId, wu.getUser().getId())) {
            scoreVerifyService.createQuery(query);
        }
        return query(query);

    }

    private PageResult<ScoreVerifyVo> query(ScoreVerifyQuery query) {
        Integer examId = getSessionExamId();
        ApiUser wu = getApiUser();
        query.setExamId(examId);
        query.setUserId(wu.getUser().getId());
        Integer totalCount = scoreVerifyService.countByQuery(query.getExamId(), query.getUserId());
        if (totalCount > 0) {
            List<ScoreVerifyVo> list = scoreVerifyService.findByQuery(query);
            query.setResult(list);
            query.setTotalCount(totalCount);
            return PageUtil.of(list, query);
        } else {
            return PageUtil.emptyPage();
        }
    }

    @ApiOperation(value = "导出标记卷")
    @RequestMapping(value = "export-flagged", method = RequestMethod.POST)
    @ResponseBody
    public void exportFile(HttpServletResponse response) {
        Integer examId = getSessionExamId();
        ApiUser wu = getApiUser();
        if (!scoreVerifyService.existByExamIdAndUserId(examId, wu.getUser().getId())) {
            throw new StatusException("没有数据需要导出");
        }
        try {
            List<ScoreVerifyVo> list = scoreVerifyService.findFlagged(examId, true, wu.getUser().getId());
            String fileName = "标记卷数据.xlsx";
            new ExportExcel("标记卷数据", ScoreVerifyVo.class).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            throw new StatusException("导出数据失败", e);
        }
    }

    @ApiOperation(value = "导出标全部")
    @RequestMapping(value = "export-all", method = RequestMethod.POST)
    @ResponseBody
    public void exportAllFile(HttpServletResponse response) {
        Integer examId = getSessionExamId();
        ApiUser wu = getApiUser();
        if (!scoreVerifyService.existByExamIdAndUserId(examId, wu.getUser().getId())) {
            throw new StatusException("没有数据需要导出");
        }
        try {
            List<ScoreVerifyVo> list = scoreVerifyService.findFlagged(examId, null, wu.getUser().getId());
            String fileName = "全部数据.xlsx";
            new ExportExcel("全部数据", ScoreVerifyVo.class).setDataList(list).write(response, fileName).dispose();
        } catch (Exception e) {
            throw new StatusException("导出数据失败", e);
        }
    }

}
